#!/bin/sh
# Update awstats database. 
#
# Doug Teodoro, Barry Blumenfeld, Lee Lueking. 4 Dec. 2006
# Modified by Dave Dykstra September 2007
# Modified by Barry Blumenfeld October 2007 to squid log reformatting and compression
# Modified by David Front 2010

AWSTATS_LOGNAME=$1

AWSTATS_ETC_DIR_=/etc/awstats
AWSTATS_DATA_DIR_BASE=/var/lib/frontier-awstats
AWSTATSSCRIPTS_DIR=/usr/share/frontier-awstats

PASSWORD_FILE=$AWSTATS_ETC_DIR_/password-file

if [ ! -s ${PASSWORD_FILE} ]; then
	echo "ERROR: password is missing: ${PASSWORD_FILE}. Leaving"
	# TBD: Take care that an error email will be sent to administrators
    exit 1
fi

AWSTATSCONF_FILE=$AWSTATS_ETC_DIR_/awstatsconf
if [ -f $AWSTATSCONF_FILE ]; then
    source $AWSTATSCONF_FILE 
fi

function getKeyFromFileWithDelimiter(){
    # $1 key
    # $2 file
    # $3 delimiter
    #echo "grep "^${1}" $2|awk -F $3 '{print $2}'"
    value=`grep "^${1}" $2|awk -F "$3" '{print $2}'`
    if [  X$value == X ]; then
        echo "ERROR: Did not find key: $1, at file: $2, with delimiter: $3"
        exit 1
    fi
    #echo "file: $1, key: $2, delimiter: $3, value: $value"
    echo $value
}

function condexport {
    eval "if [ ! \"\${${1}+x}\" ] ; then
       export ${1}=${2}
    fi"
}

function findNodeAndProject {
	mappingName=awstatsSiteProjectNodesMapping
	mappingFile=`mktemp`
	wget -O ${mappingFile}_ http://wlcg-squid-monitor.cern.ch/${mappingName}
	grep -v '#' ${mappingFile}_ | grep `dnsdomainname` > ${mappingFile}
	nodes=`awk '{print $2}' ${mappingFile}`
	#echo "nodes: ${nodes}"
	FOUND=NO
	THISNODE=`uname -n`
	for NODE in ${nodes} ; do
		if [ "$NODE" = "$THISNODE" ] ; then
			FOUND=YES
		else
			case `host $NODE 2>/dev/null` in
		    *"is an alias for $THISNODE"*)
			FOUND=YES
			;;
			esac
	    fi
	    if [ $FOUND == YES ] ; then
                AWSTATS_CONFIG=`grep ${NODE} ${mappingFile}| head -1| awk '{print $3}'`
                AWSTATS_CONFIG=${AWSTATS_CONFIG%"-httpd"}
                SITEPROJ=`grep ${NODE} ${mappingFile}| head -1| awk '{print $1}'`
                break
	    fi
	done

	if [ ! $FOUND == YES ]; then
	    if [ -n "$AWSTATS_NODENAME" ]; then
		echo "WARNING: this node was not found at the list of nodes that report to awstats"
		echo "Using the saved value"
		AWSTATS_CONFIG=$AWSTATS_NODENAME
		SITEPROJ=$AWSTATS_SITEPROJ
	    else
                /bin/rm -f ${mappingFile} ${mappingFile}_
		echo "ERROR: this node was not found at the list of nodes that report to awstats"
		exit 1
	    fi
	elif [ "$AWSTATS_NODENAME" != "$AWSTATS_CONFIG" ] || \
		[ "$AWSTATS_SITEPROJ" != "$SITEPROJ" ]; then
	    rm -f $AWSTATSCONF_FILE
	    (echo "# Automatically generated by $0 on `date`"
	    echo "export AWSTATS_NODENAME=$AWSTATS_CONFIG"
	    echo "export AWSTATS_SITEPROJ=$SITEPROJ") >$AWSTATSCONF_FILE
            source $AWSTATSCONF_FILE
	fi
        httpdMissing=NO
        if [[ $AWSTATS_LOGNAME == *"/httpd/access"* ]]; then
            if ! grep -q $AWSTATS_NODENAME-httpd ${mappingFile}; then
                httpdMissing=YES
            fi
        fi
        /bin/rm -f ${mappingFile} ${mappingFile}_
        if [ $httpdMissing == YES ]; then
            echo "$AWSTATS_NODENAME-httpd was not found in $mappingName file"
	    # This is an allowed combination for example on stratum 1s
            exit 0
        fi
}
findNodeAndProject

# Find squid number if this is for a multisquid setup, that is, the 
#  access log name ends in /squidNN.
# In order to preserve data when switching between non-multisquid and
#  multisquid, treat the first one (/squid0) almost exactly like a
#  non-multisquid setup except for a -0 appended to the upload directory
#  name, and for the others add -NN to the configuration too
AWSTATS_UPLOAD_DIR="$AWSTATS_CONFIG"
case $AWSTATS_LOGNAME in
    */squid[0-9]/access*|*squid[1-9][0-9]/access*)
	SQUIDNUM="`echo "$AWSTATS_LOGNAME"|sed 's,.*/squid,,;s,/.*,,'`"
	if [ "$SQUIDNUM" != 0 ]; then
	    AWSTATS_CONFIG="$AWSTATS_CONFIG-$SQUIDNUM"
	fi
	AWSTATS_UPLOAD_DIR="$AWSTATS_UPLOAD_DIR-$SQUIDNUM"
	;;
    */httpd/access*)
        AWSTATS_CONFIG="$AWSTATS_CONFIG-httpd"
        AWSTATS_UPLOAD_DIR="$AWSTATS_UPLOAD_DIR-httpd"
        cp -f $PASSWORD_FILE $PASSWORD_FILE.root
        PASSWORD_FILE=$PASSWORD_FILE.root
        trap "rm -f $PASSWORD_FILE" 0
        ;;
esac

CONF_FILE=$AWSTATS_ETC_DIR_/awstats.$AWSTATS_CONFIG.conf

AWSTATS_PL=$AWSTATSSCRIPTS_DIR/awstats.pl

echo "Started at `date`"

echo "Calculating awstats for $SITEPROJ-$AWSTATS_CONFIG"

# Handle awstats configuration:
# AWSTATS_DATA_DIR should be a unique directory for awstats data, otherwise, this script may rm by mistake needed files
AWSTATS_DATA_DIR=${AWSTATS_DATA_DIR_BASE}/$AWSTATS_CONFIG
#echo "AWSTATS_DATA_DIR_BASE: $AWSTATS_DATA_DIR_BASE, AWSTATS_DATA_DIR $AWSTATS_DATA_DIR"
EXTRASUBST=""
URLCOL=7
if [ -s $AWSTATS_LOGNAME ] && awk '{exit (match($1,"[.:]")>0)}' $AWSTATS_LOGNAME; then
    # There's no dot or colon in first word of the log, assume it is a syslog
    #  and ignore the first 5 words of each line
    EXTRASUBST="-e 's/^LogFormat = \"/LogFormat = \"%other %other %other %other %other /'"
    URLCOL=12
fi
eval sed -e 's,^DirData.*,DirData='${AWSTATS_DATA_DIR}',' $EXTRASUBST ${AWSTATS_ETC_DIR_}/awstats.model.conf >${CONF_FILE}.tmp
if cmp -s ${CONF_FILE}.tmp ${CONF_FILE}; then
    # no change, no need to update
    /bin/rm -f ${CONF_FILE}.tmp
else
    /bin/mv -f ${CONF_FILE}.tmp ${CONF_FILE}
fi

if [ ! -d $AWSTATS_DATA_DIR ]; then
      mkdir -p $AWSTATS_DATA_DIR
fi

# Clear DNS cache once per day
dns_cache_clear_file=${AWSTATS_DATA_DIR}/dnscacheclear.${AWSTATS_CONFIG}.txt
today=`date +%j`
if [ -f "$dns_cache_clear_file" ]; then
    dns_cache_clear_day=`cat $dns_cache_clear_file`
    if [[ "$today" != "$dns_cache_clear_day" ]]; then
        rm -f ${AWSTATS_DATA_DIR}/dnscachelastupdate.${AWSTATS_CONFIG}.txt
        echo $today > $dns_cache_clear_file
        echo "DNS cache clear at" $(date)
    fi
else
    echo $today > $dns_cache_clear_file
fi

#  Change the URL into just the servlet name and the following component,
#  because awstats can't ignore the portion after the servlet name when the
#  delimiter is a '/' instead of question mark, as is the case starting in
#  frontier_client 2.8.0.  Include the component following the servlet name
#  because that's important information for CVMFS (it's always just "Frontier"
#  for Frontier).
TMPFILE=$AWSTATS_LOGNAME.shorturl
rm -f $TMPFILE
#Extract new log lines to $TMPFILE
parsestart=${AWSTATS_DATA_DIR}/.parsestart.${AWSTATS_CONFIG}.txt
firstline=${AWSTATS_DATA_DIR}/.firstline.${AWSTATS_CONFIG}.txt
let nextbyte="`stat -c %s $AWSTATS_LOGNAME` + 1"
currentfirstline=`head -n 1 $AWSTATS_LOGNAME`

if [ -e $parsestart ]; then
  cst=`cat $parsestart`
  if [ -e $firstline ]; then
     line=`cat $firstline`
  else
     line=""
  fi
  if [ -z "$cst" ] || [ "$cst" -gt "$nextbyte" ] || [ "$line" != "$currentfirstline" ]; then
    cst=1
  fi
else
  cst=1
fi

echo $currentfirstline > $firstline

tail -c +$cst $AWSTATS_LOGNAME | awk -v cst=$cst -v parsestart=$parsestart -b '
  BEGIN { cex=0 }
  { cex += length($0) + 1 }
  { sub("http://[^/]*/","",$'$URLCOL');split($'$URLCOL',a,"/");$'$URLCOL'=a[1] "/" a[2];sub("?.*","",$'$URLCOL');print }
  END { print cst + cex > parsestart }
  ' > $TMPFILE

python3 ${AWSTATS_ETC_DIR_}/collect_pages_data.py $TMPFILE ${AWSTATS_DATA_DIR}/pages$(date '+%m%Y%d').${AWSTATS_CONFIG}.txt

#  Monthly stats
${AWSTATS_PL} -update -LogFile=${TMPFILE} -config=$AWSTATS_CONFIG -nboflastupdatelookuptosave=10000
#  Daily stats
${AWSTATS_PL} -update -LogFile=${TMPFILE} -config=$AWSTATS_CONFIG -databasebreak=day -nboflastupdatelookuptosave=10000
rm -f ${TMPFILE}
#  Delete stats files older than 20 days, keeping the others in case transfers are down for a short while
find $AWSTATS_DATA_DIR -mtime +20 | xargs -t rm -f

#  Transfer to squid monitoring server for publishing
rsync --timeout=30 --contimeout=10 --password-file=$PASSWORD_FILE -rlptv $AWSTATS_DATA_DIR/ $SITEPROJ@wlcg-squid-monitor.cern.ch::awstats-$SITEPROJ/$AWSTATS_UPLOAD_DIR

echo "Finished at `date`"
